define([
  "dojo/_base/declare",
  "dojo/string"
], function(declare, string) {
  return declare(null, {
    constructor: function() {
      this.menus = [];
      this.menuId = 1;
      this.itemId = 1;
    },

    addMenu: function(menu) {
      menu.setId("" + this.menuId++);
      this.menus.push(menu);
      return menu;
    },

    getAllMenu: function() {
      return this.menus;
    },

    getMenu: function(id) {
      for (var i=0,l=this.menus.length; i<l; i++) {
        if (id === this.menus[i].id) {
          return this.menus[i];
        }
        var menu = this.menus[i].getMenu(id);
        if (menu) {
          return menu;
        }
      }
      return null;
    },

    getMenus: function() {
      var layerMenus = [];
      for (var i=0,l=this.menus.length; i<l; i++) {
        layerMenus.push(this.menus[i]);
        layerMenus.concat(this.menus[i].getMenus());
      }
      return layerMenus;
    },

    getMenuById: function(id) {
      for (var i=0,l=this.menus.length; i<l; i++) {
        var menu = this.menus[i];
        var menuId = id.split("menuTab_tablist_tab")[1];
        if (menu.id === menuId) {
          return this.menus[i];
        }
      }
      return null;
    },

    getMenuByTitle: function(title) {
      for (var i=0,l=this.menus.length; i<l; i++) {
        if (title === this.menus[i].title) {
          return this.menus[i];
        }
      }
      return null;
    },

    getMenuId: function() {
      return this.menuId++;
    },

    getItemId: function() {
      return "" + this.itemId++;
    },

    getItem: function(id) {
      if (!id) {
        return null;
      }
      return this.getItemFromMenus(id, this.menus);
    },

    getItemFromMenus: function(id, menus) {
      var item = null;
      for (var i=0,l=menus.length; i<l; i++) {
        var menu = menus[i];
        if (!menu) {
          continue;
        }
        var items = menu.getItems();
        for (var j=0,m=items.length; j<m; j++) {
          if (id === items[j].id) {
            return items[j];
          }
        }
        item = this.getItemFromMenus(id, menu.menus);
        if (item) {
          return item;
        }
      }
      return null;
    },

    reOrderingMenu: function(tabNodes) {
      for (var i=0,l=tabNodes.length; i<l; i++) {
        var tabNode = tabNodes[i];
        var menu = this.getMenuById(tabNode.children[1].id);
        if (!menu) {
          continue;
        }
        var order = i + 1;
        menu.setOrder(order);
      }
    },

    setColor: function(id, color) {
      var item = this.getItem(id);
      if (item) {
        item.setColor(color);
        return;
      }
      var menu = this.getMenu(id);
      if (menu) {
        menu.setColor(color);
      }
    },

    setItemLabelColor: function(id, color) {
      var item = this.getItem(id);
      if (!item) {
        return;
      }
      item.setLabelColor(color);
    },

    destroyMenu: function(menuId) {
      for (var i=0,l=this.menus.length; i<l; i++) {
        if (menuId !== this.menus[i].id) {
          continue;
        }
        this.menus[i].destroyItems();
        this.menus.splice(i, 1);
        break;
      }
    },

    destroy: function() {
      for (var i=0,l=this.menus.length; i<l; i++) {
        this.menus[i].destroyItems();
      }
      this.menus = [];
    },

    getItemsOfModel: function(modelId) {
      var itemsOfModel = [];
      for (var i=0,l=this.menus.length; i<l; i++) {
        var menu = this.menus[i];
        if (!menu) {
          continue;
        }
        var items = menu.getItems();
        for (var j=0,m=items.length; j<m; j++) {
          if (parseInt(modelId) === items[j].modelId) {
            itemsOfModel.push(items[j]);
          }
        }
      }
      return itemsOfModel;
    },

    deleteItemsOfModel: function(modelId) {
      for (var i=0,l=this.menus.length; i<l; i++) {
        var menu = this.menus[i];
        if (!menu) {
          continue;
        }
        var items = menu.getItems();
        for (var j=items.length - 1; j>=0; j--) {
          if (parseInt(modelId) === items[j].modelId) {
            items.splice(j, 1);
          }
        }
      }
    },

  });
});
